"""Tests for the views of the ``multilingual_survey`` app."""
from django.conf import settings
from django.test import TestCase

from django_libs.tests.mixins import ViewRequestFactoryTestMixin
from mixer.backend.django import mixer

from .. import views


class SurveyReportAdminViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``SurveyReportAdminView`` view class."""
    view_class = views.SurveyReportAdminView

    def get_view_kwargs(self):
        return {'slug': self.survey.slug}

    def setUp(self):
        self.admin = mixer.blend('auth.User', is_staff=True)
        self.user = mixer.blend('auth.User')
        self.question = mixer.blend(
            'multilingual_survey.SurveyQuestionTranslation').master
        self.survey = self.question.survey
        self.answer = mixer.blend(
            'multilingual_survey.SurveyAnswerTranslation',
            question=self.question).master

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        # should also redirect to login for regular users
        self.redirects(user=self.user, to='{0}?next={1}'.format(
                       settings.LOGIN_URL, self.get_url()))
        self.is_callable(user=self.admin)
        self.is_callable(user=self.admin, data={'answer': 999})
        self.is_callable(user=self.admin, data={'answer': self.answer.pk})


class SurveyReportListViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``SurveyReportListView`` view class."""
    view_class = views.SurveyReportListView

    def setUp(self):
        self.admin = mixer.blend('auth.User', is_staff=True)
        self.user = mixer.blend('auth.User')
        self.survey = mixer.blend(
            'multilingual_survey.SurveyTranslation').master

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        # should also redirect to login for regular users
        self.redirects(user=self.user, to='{0}?next={1}'.format(
                       settings.LOGIN_URL, self.get_url()))
        self.is_callable(user=self.admin)


class SurveyViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``SurveyView`` view class."""
    view_class = views.SurveyView

    def get_view_kwargs(self):
        return {'slug': self.survey.slug}

    def setUp(self):
        self.survey = mixer.blend(
            'multilingual_survey.SurveyTranslation').master
        self.data = {}

    def test_view(self):
        self.is_callable(add_session=True)
        self.is_not_callable(kwargs={'slug': 'foo'})
        self.is_postable(add_session=True, data=self.data, ajax=True)



"""Templatetags for the ``multilingual_survey`` app."""
from django import template


register = template.Library()


@register.assignment_tag()
def filter_responses(obj, user_selection, session_selection):
    """Returns the respons count for a specific user selection."""
    if user_selection or session_selection:
        responses = obj.responses.filter(
            user__pk__in=user_selection) | obj.responses.filter(
                session_id__in=session_selection)
        return responses.distinct()
    return obj.responses.all()



"""Tests for the models of the multilingual_survey app."""
from django.test import TestCase

from mixer.backend.django import mixer


class SurveyTestCase(TestCase):
    """Tests for the ``Survey`` model."""
    longMessage = True

    def test_model(self):
        obj = mixer.blend('multilingual_survey.SurveyTranslation')
        self.assertTrue(obj.pk)


class SurveyQuestionTestCase(TestCase):
    """Tests for the ``SurveyQuestion`` model."""
    longMessage = True

    def test_model(self):
        obj = mixer.blend('multilingual_survey.SurveyQuestionTranslation')
        self.assertTrue(obj.pk)


class SurveyAnswerTestCase(TestCase):
    """Tests for the ``SurveyAnswer`` model."""
    longMessage = True

    def test_model(self):
        obj = mixer.blend('multilingual_survey.SurveyAnswerTranslation')
        self.assertTrue(obj.pk)


class SurveyResponseTestCase(TestCase):
    """Tests for the ``SurveyResponse`` model."""
    longMessage = True

    def test_model(self):
        obj = mixer.blend('multilingual_survey.SurveyResponse')
        self.assertTrue(obj.pk)



"""Tests for the forms of the multilingual_survey app."""
from django.contrib.auth.models import AnonymousUser
from django.test import TestCase

from mixer.backend.django import mixer

from .. import forms
from .. import models


class SurveyFormTestCase(TestCase):
    """Tests for the ``SurveyForm``."""
    longMessage = True

    def setUp(self):
        self.user = mixer.blend('auth.User')
        self.answer1 = mixer.blend(
            'multilingual_survey.SurveyAnswerTranslation',
            language_code='en').master
        self.question1 = self.answer1.question
        self.survey = self.question1.survey
        self.question2 = mixer.blend(
            'multilingual_survey.SurveyQuestionTranslation',
            survey=self.survey, has_other_field=True,
            language_code='en').master
        mixer.blend(
            'multilingual_survey.SurveyResponse',
            other_answer='Foo', question=self.question2, language_code='en',
            user=self.user)
        self.question3 = mixer.blend(
            'multilingual_survey.SurveyQuestionTranslation',
            survey=self.survey, is_multi_select=True, has_other_field=True,
            required=True, language_code='en').master
        self.answer3_1 = mixer.blend(
            'multilingual_survey.SurveyAnswerTranslation',
            language_code='en',
            question=self.question3).master
        self.answer3_2 = mixer.blend(
            'multilingual_survey.SurveyAnswerTranslation',
            language_code='en',
            question=self.question3).master
        self.question4 = mixer.blend(
            'multilingual_survey.SurveyQuestionTranslation',
            language_code='en',
            survey=self.survey).master
        self.answer4 = mixer.blend(
            'multilingual_survey.SurveyAnswerTranslation',
            language_code='en',
            question=self.question4).master

        self.data = {
            # question 1 was not required
            self.question1.slug: [],
            '{0}_other'.format(self.question2.slug): 'Foo',
            self.question3.slug: [self.answer3_1.pk],
            self.question4.slug: self.answer4.pk,
        }

    def test_form_with_user(self):
        form = forms.SurveyForm(self.user, 'foo', self.survey)
        self.assertTrue(self.question1.slug in form.fields.keys(), msg=(
            'Should dynamically add fields for all questions to the form'))
        self.assertFalse(
            self.question1.slug + '_other' in form.fields.keys(), msg=(
                'Should not add `other` field if the question has not enabled'
                ' it'))

        """
        TODO: Fix the following tests:

        self.assertTrue(self.question3.slug in form.fields.keys(), msg=(
            'Should dynamically add fields for all questions to the form'))
        self.assertTrue(
            self.question3.slug + '_other' in form.fields.keys(), msg=(
                'Should dynamically add `other` field if the question has'
                ' enabled it'))

        """

        form = forms.SurveyForm(self.user, 'foo', self.survey, self.data)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid. Errors: {0}'.format(form.errors)))

        valid_data = {
            # question 1 was not required
            self.question1.slug: [],
            '{0}_other'.format(self.question2.slug): 'Foo',
            '{0}_other'.format(self.question3.slug): 'Something',
        }
        form = forms.SurveyForm(self.user, 'foo', self.survey, valid_data)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid. Errors: {0}'.format(form.errors)))

        # one question is required and the other is not, so not passing
        # anything should make it invalid
        bad_data = {}
        form = forms.SurveyForm(self.user, 'foo', self.survey, bad_data)
        """
        TODO: Fix the following tests:

        self.assertFalse(form.is_valid(), msg='The form should not be valid.')

        """

        self.question3.has_other_field = False
        self.question3.save()
        form = forms.SurveyForm(self.user, 'foo', self.survey, self.data)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid. Errors: {0}'.format(form.errors)))

        form.save()
        """
        TODO: Fix the following tests:

        self.assertEqual(models.SurveyResponse.objects.count(), 3, msg=(
            'When saved, there should be three responses in the database.'))

        """

        valid_data = self.data.copy()
        valid_data.update({self.question1.slug: self.answer1.pk})
        form = forms.SurveyForm(self.user, 'foo', self.survey, valid_data)
        self.assertTrue(form.is_valid(), msg=(
            'The form should still be valid. Errors: {0}'.format(form.errors)))
        form.save()
        """
        TODO: Fix the following tests:

        self.assertEqual(models.SurveyResponse.objects.count(), 4, msg=(
            'When saved again with more responses, there should be 4'
            ' responses in the database.'))

        self.assertEqual(
            form.initial[self.question3.slug], [self.answer3_1.pk], msg=(
                'If no data given, the form should add the already known'
                ' answers from the database'))

        """

    def test_form_with_anonymous(self):
        form = forms.SurveyForm(AnonymousUser(), 'foo', self.survey, self.data)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid. Errors: {0}'.format(form.errors)))
        form.save()
        self.assertEqual(models.SurveyResponse.objects.count(), 1, msg=(
            'When saved, there should be 1 response in the database.'))



"""Admin classes for the multilingual_survey app."""
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from generic_positions.admin import GenericPositionsAdmin
from hvad.admin import TranslatableAdmin

from . import models


class SurveyAdmin(TranslatableAdmin):
    """Custom admin for the ``Survey`` model."""
    list_display = ['get_title', 'slug']

    def get_title(self, obj):
        return obj.__unicode__()
    get_title.short_description = _('Title')


class SurveyQuestionAdmin(GenericPositionsAdmin, TranslatableAdmin):
    """Custom admin for the ``SurveyQuestion`` model."""
    list_display = ['get_title', 'slug', 'is_multi_select',
                    'has_other_field', 'required']
    list_filter = ['survey', ]

    def get_title(self, obj):
        return obj.__unicode__()
    get_title.short_description = _('Title')


class SurveyAnswerAdmin(GenericPositionsAdmin, TranslatableAdmin):
    """Custom admin for the ``SurveyAnswer`` model."""
    list_display = ['get_title', 'slug']
    list_filter = ['question', ]

    def get_title(self, obj):
        return obj.__unicode__()
    get_title.short_description = _('Title')


class SurveyResponseAdmin(admin.ModelAdmin):
    """Custom admin for the ``SurveyResponse`` model."""
    list_display = ['user_email', 'question', 'get_answer', 'date_created']

    def get_answer(self, obj):
        answer_string = ''
        for answer in obj.answer.all():
            if answer_string == '':
                answer_string += answer.__unicode__()
            else:
                answer_string += u', {0}'.format(answer.__unicode__())
        if obj.other_answer:
            if answer_string == '':
                answer_string += obj.other_answer
            else:
                answer_string += u', *{0}'.format(obj.other_answer)
        return answer_string[:30]
    get_answer.short_description = _('Answer')

    def user_email(self, obj):
        return obj.user.email if obj.user else 'Anonymous'
    user_email.short_description = _('User')


admin.site.register(models.SurveyResponse, SurveyResponseAdmin)
admin.site.register(models.SurveyAnswer, SurveyAnswerAdmin)
admin.site.register(models.SurveyQuestion, SurveyQuestionAdmin)
admin.site.register(models.Survey, SurveyAdmin)



"""Forms for the multilingual_survey app."""
from collections import OrderedDict

from django import forms
from django.utils.translation import ugettext_lazy as _

from generic_positions.templatetags.position_tags import order_by_position

from . import models

try:
    # Django < 1.7.x
    from django.forms.util import ErrorList
except ImportError:
    # Django >= 1.7.x
    from django.forms.utils import ErrorList


class SurveyForm(forms.Form):
    """
    Form that renders a survey and saves the returned answers.

    The form needs to be built fully dynamically since we don't know the number
    of questions and answers and we also don't know the type of the answers
    (multi-select vs. single-select).

    """
    def __init__(self, user, session_key, survey, data=None, files=None,
                 auto_id='id_%s', prefix=None, initial=None, label_suffix=':',
                 error_class=ErrorList, empty_permitted=False):
        """
        Based on the given Survey, adds all necessary fields dynamically.

        Based on the given user, correctly sets initial values if the user
        has filled out this survey in the past.

        """
        self.user = user if user.is_authenticated() else None
        self.session_key = session_key
        self.survey = survey
        self.base_fields = {}
        self.is_bound = data is not None or files is not None

        self.data = data or {}
        self.files = files or {}
        self.auto_id = auto_id
        self.prefix = prefix
        self.error_class = error_class
        self.label_suffix = label_suffix
        self.empty_permitted = empty_permitted
        self._errors = None
        self._changed_data = None
        # To maintain the order of questions, that by default order by position
        # field, we use the OrderedDict for adding fields.
        self.fields = OrderedDict()
        self._bound_fields_cache = {}
        self.initial = initial or self.get_initial()

        for question in order_by_position(self.survey.questions.all()):
            # First we add the select/multiselect for the question
            queryset = question.answers.all()
            if queryset:
                field_kwargs = {
                    'label': question.__unicode__(),
                    'queryset': queryset.order_by(
                        'generic_position__position'),
                    'required': False,
                }

                if self.initial.get(question.slug):
                    field_kwargs.update({'initial': self.initial.get(
                        question.slug)})

                if question.is_multi_select:
                    field_kwargs.update({
                        'widget': forms.CheckboxSelectMultiple,
                    })
                    self.fields.update({
                        question.slug: forms.ModelMultipleChoiceField(
                            **field_kwargs)})
                else:
                    self.fields.update({
                        question.slug: forms.ModelChoiceField(**field_kwargs)})

                # Then we add the `other` field for the question
                if question.has_other_field:
                    field_name = u'{0}_other'.format(question.slug)
                    self.fields.update({
                        field_name: forms.CharField(
                            label=_('Other'), max_length=2014,
                            required=False)})
                    self.fields[field_name].widget.attrs.update({
                        'data-class': 'other-field'})
            elif question.has_other_field:
                self.fields.update({
                    u'{0}_other'.format(question.slug): forms.CharField(
                        label=question.__unicode__(), max_length=2014,
                        required=question.required,
                        initial=self.initial.get(question.slug))})

    def get_initial(self):
        initial = {}
        for question in self.survey.questions.all():
            if self.user:
                try:
                    response = self.user.responses.filter(
                        question=question).distinct().get()
                except models.SurveyResponse.DoesNotExist:
                    continue
            else:
                try:
                    response = models.SurveyResponse.objects.filter(
                        question=question,
                        session_id=self.session_key,
                        user__isnull=True,
                    ).distinct().get()
                except models.SurveyResponse.DoesNotExist:
                    continue
            if not response.other_answer:
                if question.is_multi_select:
                    initial[question.slug] = [
                        resp.pk for resp in response.answer.all()]
                elif response.answer.all():
                    initial[question.slug] = response.answer.all()[0].pk
            else:
                initial[question.slug] = response.other_answer
        return initial

    def clean(self):
        for question in self.survey.questions.all():
            if question.required:
                response = self.cleaned_data.get(question.slug)
                if not response and (
                        not question.has_other_field or
                        not self.cleaned_data.get(
                        '{0}_other'.format(question.slug))):
                    self._errors[question.slug] = [_(
                        'This field is required.')]
        return self.cleaned_data

    def save(self):
        for question in self.survey.questions.all():
            # read the response from the cleaned data
            response = self.cleaned_data.get(question.slug)
            # if there is none but there is an other field, try again
            other_response = False
            if question.has_other_field:
                other_response = self.cleaned_data.get('{0}_other'.format(
                    question.slug))

            # if there was no response given in the data, remove the old one
            # and continue
            if not response and not other_response:
                if self.user:
                    models.SurveyResponse.objects.filter(
                        user=self.user,
                        question=question,
                    ).delete()
                else:
                    models.SurveyResponse.objects.filter(
                        user__isnull=True,
                        session_id=self.session_key,
                        question=question,
                    ).delete()
                continue

            # otherwise check if there was a response. If not create one.
            if self.user:
                resp_obj, crtd = models.SurveyResponse.objects.get_or_create(
                    user=self.user, question=question)
            else:
                resp_obj, crtd = models.SurveyResponse.objects.get_or_create(
                    user__isnull=True, session_id=self.session_key,
                    question=question)

            # Assign the answer to the user response object
            resp_obj.answer.clear()
            resp_obj.other_answer = ''
            if other_response:
                resp_obj.other_answer = other_response
            if response:
                if isinstance(response, models.SurveyAnswer):
                    resp_obj.answer.add(response)
                else:
                    for answer in response:
                        resp_obj.answer.add(answer)
            resp_obj.save()
        return self.survey



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-03-02 02:16
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_libs.models_mixins


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Survey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=256, unique=True, verbose_name='Slug')),
            ],
            options={
                'abstract': False,
            },
            bases=(django_libs.models_mixins.TranslationModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SurveyAnswer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=256, verbose_name='Slug')),
            ],
            bases=(django_libs.models_mixins.TranslationModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SurveyAnswerTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256, verbose_name='Title')),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='multilingual_survey.SurveyAnswer')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'multilingual_survey_surveyanswer_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='SurveyQuestion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=256, verbose_name=b'Slug')),
                ('is_multi_select', models.BooleanField(default=False, verbose_name='Is multi-select')),
                ('has_other_field', models.BooleanField(default=False, verbose_name='Has other-field')),
                ('required', models.BooleanField(default=False, verbose_name='Required')),
                ('survey', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='multilingual_survey.Survey', verbose_name='Survey')),
            ],
            bases=(django_libs.models_mixins.TranslationModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SurveyQuestionTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256, verbose_name='Title')),
                ('content', models.TextField(blank=True, verbose_name='Content')),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='multilingual_survey.SurveyQuestion')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'multilingual_survey_surveyquestion_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='SurveyResponse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('session_id', models.CharField(blank=True, max_length=1024, verbose_name='Session ID')),
                ('other_answer', models.CharField(blank=True, max_length=1024, verbose_name='Other answer')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('answer', models.ManyToManyField(related_name='responses', to='multilingual_survey.SurveyAnswer', verbose_name='Answer')),
                ('question', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='responses', to='multilingual_survey.SurveyQuestion', verbose_name='Question')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='responses', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'ordering': ('question',),
            },
        ),
        migrations.CreateModel(
            name='SurveyTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256, verbose_name='Title')),
                ('description', models.TextField(blank=True, max_length=2048, verbose_name='Description')),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='multilingual_survey.Survey')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'multilingual_survey_survey_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.AddField(
            model_name='surveyanswer',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='multilingual_survey.SurveyQuestion', verbose_name='Question'),
        ),
        migrations.AlterUniqueTogether(
            name='surveytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='surveyquestiontranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='surveyquestion',
            unique_together=set([('slug', 'survey')]),
        ),
        migrations.AlterUniqueTogether(
            name='surveyanswertranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='surveyanswer',
            unique_together=set([('slug', 'question')]),
        ),
    ]



"""URLs for the multilingual_survey app."""
from django.conf.urls import url

from . import views


urlpatterns = [
    url(r'^report/(?P<slug>[\w-]+)/$',
        views.SurveyReportAdminView.as_view(),
        name='multilingual_survey_admin_report'),
    url(r'^report/$',
        views.SurveyReportListView.as_view(),
        name='multilingual_survey_list_report'),
    url(r'^(?P<slug>[\w-]+)/$',
        views.SurveyView.as_view(),
        name='multilingual_survey_detail'),
]



"""Models for the multilingual_survey app"""
from django.contrib.contenttypes import fields
from django.db import models
from django.utils.translation import ugettext_lazy as _

from django_libs.models_mixins import TranslationModelMixin
from hvad.models import TranslatableModel, TranslatedFields


class Survey(TranslationModelMixin, TranslatableModel):
    """
    A Survey consists of several Questions.

    :title: The name of this survey. Authors may change the title.
    :description: Optional description to add extra information to the survey.
    :slug: The slug of this survey. The slug should never be changed, since it
      might be referenced in the code.

    """
    translations = TranslatedFields(
        title=models.CharField(
            verbose_name=_('Title'),
            max_length=256,
        ),
        description=models.TextField(
            verbose_name=_('Description'),
            max_length=2048,
            blank=True,
        )
    )

    slug = models.SlugField(
        verbose_name=_('Slug'),
        max_length=256,
        unique=True,
    )


class SurveyQuestion(TranslationModelMixin, TranslatableModel):
    """
    Belongs to a Survey and has several SurveyAnswers.

    :title: The title of this question.
    :slug: The slug of this question. This will be used to create the form's
      field name.
    :content: An optional longer description of this question.
    :survey: FK to Survey.
    :is_multi_select: If ``True``, we will render checkboxes instead of
      radiobuttons or a drop-down-list..
    :has_other_field: If ``True``, the SurveyForm will allow the user to input
      any value into a "other" field. If ``False``, no such field will be
      rendered.
    :required: Makes the field required, but accepts either a selected answer
      or the other field, if there is one.
    :position: Can be used to order questions in a survey.

    """
    translations = TranslatedFields(
        title=models.CharField(
            verbose_name=_('Title'),
            max_length=256,
        ),
        content=models.TextField(
            verbose_name=_('Content'),
            blank=True,
        )
    )

    slug = models.SlugField(
        verbose_name=('Slug'),
        max_length=256,
    )

    survey = models.ForeignKey(
        Survey,
        verbose_name=_('Survey'),
        related_name='questions',
    )

    is_multi_select = models.BooleanField(
        verbose_name=_('Is multi-select'),
        default=False,
    )

    has_other_field = models.BooleanField(
        verbose_name=_('Has other-field'),
        default=False,
    )

    required = models.BooleanField(
        verbose_name=_('Required'),
        default=False,
    )

    generic_position = fields.GenericRelation(
        'generic_positions.ObjectPosition'
    )

    class Meta:
        unique_together = ('slug', 'survey')


class SurveyAnswer(TranslationModelMixin, TranslatableModel):
    """
    Belongs to a SurveyQuestion.

    :title: The title of this answer. Authors may change this title.
    :slug: The slug of this answer. Should never be changed since it might be
      referenced in the code.
    :position: Can be used to order answers.

    """
    translations = TranslatedFields(
        title=models.CharField(verbose_name=_('Title'), max_length=256),
    )

    slug = models.SlugField(
        verbose_name=_('Slug'),
        max_length=256,
    )

    question = models.ForeignKey(
        SurveyQuestion,
        verbose_name=_('Question'),
        related_name='answers'
    )

    generic_position = fields.GenericRelation(
        'generic_positions.ObjectPosition'
    )

    class Meta:
        unique_together = ('slug', 'question')


class SurveyResponse(models.Model):
    """
    Ties a user response to an answer.

    :user: Optional FK to the User. If ``None``, we are dealing with an
      anonymous answer.
    :session_id: Optional session id storage to identify anonymous users.
    :question: Optional FK to a SurveyQuestion. Must be set, if ``answer`` is
      not set but ``other_answer`` is set, so that we know to which question
      this custom answer belongs.
    :answer: Optional FK to a SurveyAnswer. If ``None``, then ``other_answer``
      must be given.
    :other_answer: Optional free text entered by the user if no available
      answer matches him. If ``None``, then ``answer`` must be given.
    :date_created: Creation date of this answer.

    """
    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('User'),
        related_name='responses',
        blank=True, null=True,
    )

    session_id = models.CharField(
        verbose_name=_('Session ID'),
        max_length=1024,
        blank=True,
    )

    question = models.ForeignKey(
        SurveyQuestion,
        verbose_name=_('Question'),
        related_name='responses',
        blank=True, null=True,
    )

    answer = models.ManyToManyField(
        SurveyAnswer,
        verbose_name=_('Answer'),
        related_name='responses',
    )

    other_answer = models.CharField(
        verbose_name=_('Other answer'),
        max_length=1024,
        blank=True,
    )

    date_created = models.DateTimeField(
        verbose_name=_('Date created'),
        auto_now_add=True,
    )

    def __unicode__(self):
        return u'Answer to {0} from {1}'.format(
            self.question, self.user.email if self.user else '(?)')

    class Meta:
        ordering = ('question', )



"""Views for the multilingual_survey app."""
from django.views.generic import DetailView, FormView, ListView
from django.contrib.auth.decorators import user_passes_test
from django.http import Http404
from django.utils.decorators import method_decorator

from . import forms
from . import models


class SurveyReportAdminView(DetailView):
    """A view to display results of a survey for admins."""
    model = models.Survey
    template_name = 'multilingual_survey/survey_report_admin.html'

    @method_decorator(user_passes_test(lambda u: u.is_staff))
    def dispatch(self, request, *args, **kwargs):
        if request.GET.get('answer'):
            try:
                self.answer = models.SurveyAnswer.objects.get(
                    pk=request.GET['answer'])
            except models.SurveyAnswer.DoesNotExist:
                pass
        return super(SurveyReportAdminView, self).dispatch(
            request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(SurveyReportAdminView, self).get_context_data(**kwargs)
        if hasattr(self, 'answer') and self.answer:
            context.update({
                'user_selection': self.answer.responses.values_list(
                    'user__pk', flat=True),
                'session_selection': self.answer.responses.values_list(
                    'session_id', flat=True),
                'current_answer': self.answer,
            })
        return context


class SurveyReportListView(ListView):
    """A view to display a list of surveys for admins."""
    model = models.Survey
    template_name = 'multilingual_survey/survey_report_list.html'

    @method_decorator(user_passes_test(lambda u: u.is_staff))
    def dispatch(self, request, *args, **kwargs):
        return super(SurveyReportListView, self).dispatch(
            request, *args, **kwargs)


class SurveyView(FormView):
    """A view to display a survey."""
    form_class = forms.SurveyForm
    template_name = 'multilingual_survey/survey_detail.html'

    def dispatch(self, request, *args, **kwargs):
        try:
            self.survey = models.Survey.objects.get(slug=kwargs['slug'])
        except models.Survey.DoesNotExist:
            raise Http404
        return super(SurveyView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(SurveyView, self).get_context_data(**kwargs)
        context.update({'survey': self.survey})
        return context

    def get_form_kwargs(self):
        kwargs = super(SurveyView, self).get_form_kwargs()
        kwargs.update({
            'user': self.request.user,
            'survey': self.survey,
            'session_key': self.request.session.session_key,
        })
        return kwargs

    def form_valid(self, form):
        self.object = form.save()
        context = self.get_context_data(form=form)
        context.update({'success': True})
        return self.render_to_response(context)



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SurveyTranslation'
        db.create_table(u'multilingual_survey_survey_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['multilingual_survey.Survey'])),
        ))
        db.send_create_signal(u'multilingual_survey', ['SurveyTranslation'])

        # Adding unique constraint on 'SurveyTranslation', fields ['language_code', 'master']
        db.create_unique(u'multilingual_survey_survey_translation', ['language_code', 'master_id'])

        # Adding model 'Survey'
        db.create_table(u'multilingual_survey_survey', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=256)),
        ))
        db.send_create_signal(u'multilingual_survey', ['Survey'])

        # Adding model 'SurveyQuestionTranslation'
        db.create_table(u'multilingual_survey_surveyquestion_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['multilingual_survey.SurveyQuestion'])),
        ))
        db.send_create_signal(u'multilingual_survey', ['SurveyQuestionTranslation'])

        # Adding unique constraint on 'SurveyQuestionTranslation', fields ['language_code', 'master']
        db.create_unique(u'multilingual_survey_surveyquestion_translation', ['language_code', 'master_id'])

        # Adding model 'SurveyQuestion'
        db.create_table(u'multilingual_survey_surveyquestion', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=256)),
            ('survey', self.gf('django.db.models.fields.related.ForeignKey')(related_name='questions', to=orm['multilingual_survey.Survey'])),
            ('is_multi_select', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('has_other_field', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('required', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('position', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal(u'multilingual_survey', ['SurveyQuestion'])

        # Adding unique constraint on 'SurveyQuestion', fields ['slug', 'survey']
        db.create_unique(u'multilingual_survey_surveyquestion', ['slug', 'survey_id'])

        # Adding model 'SurveyAnswerTranslation'
        db.create_table(u'multilingual_survey_surveyanswer_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['multilingual_survey.SurveyAnswer'])),
        ))
        db.send_create_signal(u'multilingual_survey', ['SurveyAnswerTranslation'])

        # Adding unique constraint on 'SurveyAnswerTranslation', fields ['language_code', 'master']
        db.create_unique(u'multilingual_survey_surveyanswer_translation', ['language_code', 'master_id'])

        # Adding model 'SurveyAnswer'
        db.create_table(u'multilingual_survey_surveyanswer', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=256)),
            ('question', self.gf('django.db.models.fields.related.ForeignKey')(related_name='answers', to=orm['multilingual_survey.SurveyQuestion'])),
            ('position', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal(u'multilingual_survey', ['SurveyAnswer'])

        # Adding unique constraint on 'SurveyAnswer', fields ['slug', 'question']
        db.create_unique(u'multilingual_survey_surveyanswer', ['slug', 'question_id'])

        # Adding model 'SurveyResponse'
        db.create_table(u'multilingual_survey_surveyresponse', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='responses', null=True, to=orm['auth.User'])),
            ('question', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='responses', null=True, to=orm['multilingual_survey.SurveyQuestion'])),
            ('other_answer', self.gf('django.db.models.fields.CharField')(max_length=1024, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'multilingual_survey', ['SurveyResponse'])

        # Adding M2M table for field answer on 'SurveyResponse'
        m2m_table_name = db.shorten_name(u'multilingual_survey_surveyresponse_answer')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('surveyresponse', models.ForeignKey(orm[u'multilingual_survey.surveyresponse'], null=False)),
            ('surveyanswer', models.ForeignKey(orm[u'multilingual_survey.surveyanswer'], null=False))
        ))
        db.create_unique(m2m_table_name, ['surveyresponse_id', 'surveyanswer_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'SurveyAnswer', fields ['slug', 'question']
        db.delete_unique(u'multilingual_survey_surveyanswer', ['slug', 'question_id'])

        # Removing unique constraint on 'SurveyAnswerTranslation', fields ['language_code', 'master']
        db.delete_unique(u'multilingual_survey_surveyanswer_translation', ['language_code', 'master_id'])

        # Removing unique constraint on 'SurveyQuestion', fields ['slug', 'survey']
        db.delete_unique(u'multilingual_survey_surveyquestion', ['slug', 'survey_id'])

        # Removing unique constraint on 'SurveyQuestionTranslation', fields ['language_code', 'master']
        db.delete_unique(u'multilingual_survey_surveyquestion_translation', ['language_code', 'master_id'])

        # Removing unique constraint on 'SurveyTranslation', fields ['language_code', 'master']
        db.delete_unique(u'multilingual_survey_survey_translation', ['language_code', 'master_id'])

        # Deleting model 'SurveyTranslation'
        db.delete_table(u'multilingual_survey_survey_translation')

        # Deleting model 'Survey'
        db.delete_table(u'multilingual_survey_survey')

        # Deleting model 'SurveyQuestionTranslation'
        db.delete_table(u'multilingual_survey_surveyquestion_translation')

        # Deleting model 'SurveyQuestion'
        db.delete_table(u'multilingual_survey_surveyquestion')

        # Deleting model 'SurveyAnswerTranslation'
        db.delete_table(u'multilingual_survey_surveyanswer_translation')

        # Deleting model 'SurveyAnswer'
        db.delete_table(u'multilingual_survey_surveyanswer')

        # Deleting model 'SurveyResponse'
        db.delete_table(u'multilingual_survey_surveyresponse')

        # Removing M2M table for field answer on 'SurveyResponse'
        db.delete_table(db.shorten_name(u'multilingual_survey_surveyresponse_answer'))


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'multilingual_survey.survey': {
            'Meta': {'object_name': 'Survey'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '256'})
        },
        u'multilingual_survey.surveyanswer': {
            'Meta': {'ordering': "('position',)", 'unique_together': "(('slug', 'question'),)", 'object_name': 'SurveyAnswer'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'answers'", 'to': u"orm['multilingual_survey.SurveyQuestion']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '256'})
        },
        u'multilingual_survey.surveyanswertranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'SurveyAnswerTranslation', 'db_table': "u'multilingual_survey_surveyanswer_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['multilingual_survey.SurveyAnswer']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'multilingual_survey.surveyquestion': {
            'Meta': {'ordering': "('position',)", 'unique_together': "(('slug', 'survey'),)", 'object_name': 'SurveyQuestion'},
            'has_other_field': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_multi_select': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '256'}),
            'survey': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'questions'", 'to': u"orm['multilingual_survey.Survey']"})
        },
        u'multilingual_survey.surveyquestiontranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'SurveyQuestionTranslation', 'db_table': "u'multilingual_survey_surveyquestion_translation'"},
            'content': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['multilingual_survey.SurveyQuestion']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'multilingual_survey.surveyresponse': {
            'Meta': {'ordering': "('question__position',)", 'object_name': 'SurveyResponse'},
            'answer': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['multilingual_survey.SurveyAnswer']", 'null': 'True', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'other_answer': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'blank': 'True'}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'responses'", 'null': 'True', 'to': u"orm['multilingual_survey.SurveyQuestion']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'responses'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'multilingual_survey.surveytranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'SurveyTranslation', 'db_table': "u'multilingual_survey_survey_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['multilingual_survey.Survey']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        }
    }

    complete_apps = ['multilingual_survey']



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'SurveyResponse.session_id'
        db.add_column(u'multilingual_survey_surveyresponse', 'session_id',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=1024, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'SurveyResponse.session_id'
        db.delete_column(u'multilingual_survey_surveyresponse', 'session_id')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'multilingual_survey.survey': {
            'Meta': {'object_name': 'Survey'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '256'})
        },
        u'multilingual_survey.surveyanswer': {
            'Meta': {'unique_together': "(('slug', 'question'),)", 'object_name': 'SurveyAnswer'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'answers'", 'to': u"orm['multilingual_survey.SurveyQuestion']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '256'})
        },
        u'multilingual_survey.surveyanswertranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'SurveyAnswerTranslation', 'db_table': "u'multilingual_survey_surveyanswer_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['multilingual_survey.SurveyAnswer']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'multilingual_survey.surveyquestion': {
            'Meta': {'unique_together': "(('slug', 'survey'),)", 'object_name': 'SurveyQuestion'},
            'has_other_field': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_multi_select': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '256'}),
            'survey': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'questions'", 'to': u"orm['multilingual_survey.Survey']"})
        },
        u'multilingual_survey.surveyquestiontranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'SurveyQuestionTranslation', 'db_table': "u'multilingual_survey_surveyquestion_translation'"},
            'content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['multilingual_survey.SurveyQuestion']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'multilingual_survey.surveyresponse': {
            'Meta': {'ordering': "('question',)", 'object_name': 'SurveyResponse'},
            'answer': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'responses'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['multilingual_survey.SurveyAnswer']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'other_answer': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'blank': 'True'}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'responses'", 'null': 'True', 'to': u"orm['multilingual_survey.SurveyQuestion']"}),
            'session_id': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'responses'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'multilingual_survey.surveytranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'SurveyTranslation', 'db_table': "u'multilingual_survey_survey_translation'"},
            'description': ('django.db.models.fields.TextField', [], {'max_length': '2048', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['multilingual_survey.Survey']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        }
    }

    complete_apps = ['multilingual_survey']


"""Tests for the templatetags of the ``multilingual_survey`` app."""
from django.test import TestCase

from mixer.backend.django import mixer

from ..templatetags import survey_tags


class FilterResponsesTestCase(TestCase):
    """Tests for the ``filter_responses`` tamplatetag."""
    longMessage = True

    def test_tag(self):
        answer = mixer.blend(
            'multilingual_survey.SurveyAnswerTranslation').master
        response_1 = mixer.blend('multilingual_survey.SurveyResponse',
                                 question=answer.question)
        response_1.answer.add(answer)
        response_2 = mixer.blend('multilingual_survey.SurveyResponse',
                                 question=answer.question)
        response_2.answer.add(answer)
        self.assertEqual(
            survey_tags.filter_responses(answer, [], []).count(), 2,
            msg=('Should return two responses.'))
        self.assertEqual(
            survey_tags.filter_responses(
                answer, [], [response_2.session_id]).count(),
            1, msg=('Should return one response.'))
